#!/bin/sh

# This script generates openSUSE images for OpenNebula.
#
# Run locally (without network) with:
#   qemu-system-x86_64 -enable-kvm -m 1G -drive file=$IMAGE,format=qcow2

set -e
set -x

# XXX: Handle command-line arguments?
RELEASE=leap
RELEASE_VERSION=15.3
IMAGE_PATH=opensuse-${RELEASE}${RELEASE_VERSION}-$(date -I).img.qcow2
IMAGE_SIZE=10G
NBD_DEVICE=/dev/nbd1

# TODO: find the package definition and built ourself, publish in some RPM repository.
ONE_CONTEXT_RPM_URL="https://github.com/OpenNebula/addon-context-linux/releases/download/v5.12.0.2/one-context-5.12.0.2-1.suse.noarch.rpm"
ONE_CONTEXT_RPM_PATH=/root/one-context.rpm

cleanup() {
	# The order here is important.
	umount /mnt/dev/pts 2>/dev/null || true
	umount /mnt/dev/shm 2>/dev/null || true
	umount /mnt/dev 2>/dev/null || true
	umount /mnt/proc 2>/dev/null || true
	umount /mnt/run 2>/dev/null || true
	umount /mnt/sys 2>/dev/null || true
	umount /mnt/boot 2>/dev/null || true
	umount /mnt 2>/dev/null || true
	qemu-nbd --disconnect "$NBD_DEVICE" || true
}

run_root() {
	chroot /mnt /usr/bin/env \
		PATH=/sbin:/usr/sbin:/bin:/usr/bin \
		sh -c "$*"
}

if [ "$(whoami)" != 'root' ]; then
	echo "This script must be run as root." >&2
	exit 1
fi

if [ "$(lsb_release --short --id)" != "openSUSE" ]; then
	echo "WARNING: this script has been designed to run on an openSUSE system." >&2
	echo "WARNING: Not running openSUSE. Giving you 5 seconds to abort." >&2
	sleep 5
fi

repo_addr=http://download.opensuse.org
case "$RELEASE" in
	leap)
		distribution_slice=$RELEASE/$RELEASE_VERSION
		oss_repo_url="${repo_addr}/distribution/${distribution_slice}/repo/oss"
		;;
	tumbleweed)
		distribution_slice=$RELEASE
		oss_repo_url="${repo_addr}/${distribution_slice}/repo/oss"
	;;
	*)
		echo "Unkown openSUSE release: $RELEASE." >&2
		exit 1
	;;
esac
oss_update_repo_url="${repo_addr}/update/${distribution_slice}/oss"

# Create base QCOW2 image.
qemu-img create -f qcow2 "$IMAGE_PATH" "$IMAGE_SIZE"
modprobe nbd max_part=16
qemu-nbd --connect="$NBD_DEVICE" "$IMAGE_PATH"

# Don't forget to cleanup, even if the script crash.
trap cleanup EXIT

# Create partition table, format partitions.
sfdisk --no-reread "$NBD_DEVICE" <<EOF
1M,500M,L,*
,,L
EOF

mkfs.ext4 "${NBD_DEVICE}p1"
mkfs.ext4 "${NBD_DEVICE}p2"

# Mount partitions, install base OS.
mount "${NBD_DEVICE}p2" /mnt
mkdir /mnt/boot
mount "${NBD_DEVICE}p1" /mnt/boot

zypper --root /mnt addrepo $oss_repo_url opensuse-$RELEASE-oss
zypper --root /mnt addrepo $oss_update_repo_url opensuse-$RELEASE-oss-update
zypper --root /mnt --gpg-auto-import-keys refresh
zypper --root /mnt --non-interactive install -t pattern base

mount --bind /dev /mnt/dev
mount --bind /dev/pts /mnt/dev/pts
mount --bind /dev/shm /mnt/dev/shm
mount --bind /proc /mnt/proc
mount --bind /run /mnt/run
mount --bind /sys /mnt/sys

# Guest networking is to be handled by the one-context package.
# See https://github.com/OpenNebula/addon-context-linux for details.
# Note: as of writing, one-context does not support NetworkManager or
# systemd-networkd.

# Required to resolve package mirror in chroot.
#cat /etc/resolv.conf /mnt/etc/resolv.conf

# Initialize /etc/hosts.
cat > /mnt/etc/hosts << EOF
127.0.0.1   localhost localhost.localdomain localhost4 localhost4.localdomain4
::1         localhost localhost.localdomain localhost6 localhost6.localdomain6

EOF

# Install (magic?) one-context RPM and hope things works as expected.
curl -L "$ONE_CONTEXT_RPM_URL" > "/mnt$ONE_CONTEXT_RPM_PATH"
run_root zypper -n --no-gpg-checks install "$ONE_CONTEXT_RPM_PATH"
run_root rm "$ONE_CONTEXT_RPM_PATH"

# Install resize2fs, which is required to resize the root file-system.
run_root zypper -n install e2fsprogs

# Initalize base services.
run_root systemd-machine-id-setup

run_root ln -sf /usr/share/zoneinfo/UTC /etc/localtime
run_root systemctl enable systemd-timesyncd.service

# Install haveged due to lack of entropy in ONE environment.
run_root zypper -n install haveged
run_root systemctl enable haveged.service

# Install kernel and bootloader.
run_root zypper -n install kernel-default grub2

# Add support for virtio block devices at boot time.
cat > /mnt/etc/dracut.conf.d/virtio-blk.conf <<EOF
add_drivers="virtio-blk"
EOF
kernel_version=$(ls /mnt/boot | grep "^vmlinuz-.*" | cut -d- -f2-)
run_root dracut --force --kver $kernel_version

# Configure grub2.
run_root grub2-install --target=i386-pc "${NBD_DEVICE}"
run_root grub2-mkconfig -o /boot/grub2/grub.cfg

# Install en configure SSH daemon.
run_root zypper -n install openssh
run_root systemctl enable sshd

# Generate fstab file.
boot_uuid=$(blkid --match-tag UUID --output value "${NBD_DEVICE}p1")
root_uuid=$(blkid --match-tag UUID --output value "${NBD_DEVICE}p2")
cat >>/mnt/etc/fstab <<EOF
UUID=$boot_uuid /boot ext4 rw,relatime,data=ordered 0 2
UUID=$root_uuid /     ext4 rw,relatime,data=ordered 0 1
EOF

# Reset systemd's environment.
run_root rm -f /etc/machine-id
run_root touch /etc/machine-id
rm -f /var/lib/systemd/random-seed

# Remove temporary files and reclaim freed disk space.
# Note: build logs could be removed as well.
run_root zypper clean --all

# Make sure everything is written to disk before exiting.
sync
